/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.integration.dsl.channel;

import java.util.Comparator;

import org.springframework.integration.channel.PriorityChannel;
import org.springframework.messaging.Message;

/**
 * @author Artem Bilan
 */
public class PriorityChannelSpec extends MessageChannelSpec<PriorityChannelSpec, PriorityChannel> {

	private int capacity;

	Comparator<Message<?>> comparator;

	public PriorityChannelSpec setCapacity(int capacity) {
		this.capacity = capacity;
		return this;
	}

	public PriorityChannelSpec setComparator(Comparator<Message<?>> comparator) {
		this.comparator = comparator;
		return this;
	}

	@Override
	protected PriorityChannel doGet() {
		this.channel = new PriorityChannel(this.capacity, this.comparator);
		return super.doGet();
	}


	PriorityChannelSpec() {
	}
}
